@@ -4,18 +4,18 @@ use strict ;
 use warnings;
 use bytes;
 
-use IO::Compress::Base::Common  2.081 qw(:Status createSelfTiedObject);
+use IO::Compress::Base::Common  2.083 qw(:Status createSelfTiedObject);
 
-use IO::Uncompress::Base  2.081 ;
-use IO::Uncompress::Adapter::LZO  2.081 ;
+use IO::Uncompress::Base  2.083 ;
+use IO::Uncompress::Adapter::LZO  2.083 ;
 use Compress::LZO qw(crc32 adler32);
-use IO::Compress::Lzop::Constants  2.081 ;
+use IO::Compress::Lzop::Constants  2.083 ;
 
 
 require Exporter ;
 our ($VERSION, @ISA, @EXPORT_OK, %EXPORT_TAGS, $UnLzopError);
 
-$VERSION = '2.081';
+$VERSION = '2.083';
 $UnLzopError = '';
 
 @ISA    = qw( IO::Uncompress::Base Exporter );
@@ -503,10 +503,7 @@ This parameter defaults to 0.
 
 =item C<< BinModeOut => 0|1 >>
 
-When writing to a file or filehandle, set C<binmode> before writing to the
-file.
-
-Defaults to 0.
+This option is now a no-op. All files will be written  in binmode.
 
 =item C<< Append => 0|1 >>
 
