@@ -16,12 +16,24 @@ BEGIN {
     $extra = 1
         if eval { require Test::NoWarnings ;  import Test::NoWarnings; 1 };
 
-    plan tests => 970 + $extra ;
+    plan tests => 1007 + $extra ;
 
     use_ok('IO::Uncompress::AnyUncompress', qw(anyuncompress $AnyUncompressError)) ;
 
 }
 
+my $OriginalContent1 = <<EOM ;
+Lorem ipsum dolor sit amet, consectetuer adipiscing elit. Ut tempus odio id
+ dolor. Camelus perlus.  Larrius in lumen numen.  Dolor en quiquum filia
+ est.  Quintus cenum parat.
+EOM
+
+my $OriginalContent2 = <<EOM ;
+LOREM ipsum dolor sit amet, consectetuer adipiscing elit. Ut tempus odio id
+ dolor. Camelus perlus.  Larrius in lumen numen.  Dolor en quiquum filia
+ est.  Quintus cenum PARAT.
+EOM
+
 sub run
 {
 
@@ -42,6 +54,9 @@ sub run
         my $Func = getTopFuncRef($bit);
         my $TopType = getTopFuncName($bit);
 
+        #my $inverse = getInverse($bit);
+        #my $InverseFunc = getTopFuncRef($inverse);
+
         title "Testing $TopType Error Cases";
 
         my $a;
@@ -51,7 +66,7 @@ sub run
         like $@, mkErr("^$TopType: unknown key value\\(s\\) Fred"), '  Illegal Parameters';
 
         eval { $a = $Func->() ;} ;
-        like $@, mkErr("^$TopType: expected at least 1 parameters"), '  No Parameters';
+        like $@, "/^$TopType: expected at least 1 parameters/", '  No Parameters';
 
         eval { $a = $Func->(\$x, \1) ;} ;
         like $$Error, "/^$TopType: output buffer is read-only/", '  Output is read-only' ;
@@ -76,18 +91,18 @@ sub run
         }
 
         {
-            my $dir = "tmpdir";
+            my $dir ;
             my $lex = new LexDir $dir ;
-            mkdir $dir, 0777 ;
+            my $d = quotemeta $dir;
 
-            $a = $Func->($dir, \$x) ;
+            $a = $Func->("$dir", \$x) ;
             is $a, undef, "  $TopType returned undef";
-            like $$Error, "/input file '$dir' is a directory/",
+            like $$Error, "/input file '$d' is a directory/",
                 '  Input filename is a directory';
 
-            $a = $Func->(\$x, $dir) ;
+            $a = $Func->(\$x, "$dir") ;
             is $a, undef, "  $TopType returned undef";
-            like $$Error, "/output file '$dir' is a directory/",
+            like $$Error, "/output file '$d' is a directory/",
                 '  Output filename is a directory';
         }
 
@@ -95,14 +110,24 @@ sub run
         like $@, mkErr("^$TopType: input and output buffer are identical"),
             '  Input and Output buffer are the same';
             
-        my $lex = new LexFile my $out_file ;
-        open OUT, ">$out_file" ;
-        eval { $a = $Func->(\*OUT, \*OUT) ;} ;
-        like $@, mkErr("^$TopType: input and output handle are identical"),
-            '  Input and Output handle are the same';
-            
-        close OUT;
-        is -s $out_file, 0, "  File zero length" ;
+        SKIP:
+        {
+            # Threaded 5.6.x seems to have a problem comparing filehandles.
+            use Config;
+
+            skip 'Cannot compare filehandles with threaded $]', 2
+                if $] >= 5.006  && $] < 5.007 && $Config{useithreads};
+
+            my $lex = new LexFile my $out_file ;
+            open OUT, ">$out_file" ;
+            eval { $a = $Func->(\*OUT, \*OUT) ;} ;
+            like $@, mkErr("^$TopType: input and output handle are identical"),
+                '  Input and Output handle are the same';
+                
+            close OUT;
+            is -s $out_file, 0, "  File zero length" ;
+        }
+
         {
             my %x = () ;
             my $object = bless \%x, "someClass" ;
@@ -165,13 +190,24 @@ sub run
             my $out ;
             my @x ;
 
-            eval { $a = $Func->(\$in, \$out, TrailingData => \"abc") ;} ;
-            like $@, mkErr("^$TopType: Parameter 'TrailingData' not writable"),
-                '  TrailingData output not writable';
+            SKIP:
+            {
+                use Config;
+
+                skip 'readonly + threads', 1
+                    if $Config{useithreads} ;
+
+                skip '\\ returns mutable value in 5.19.3', 1
+                    if $] >= 5.019003;
+                
+                eval { $a = $Func->(\$in, \$out, TrailingData => \"abc") ;} ;
+                like $@, mkErr("^$TopType: Parameter 'TrailingData' not writable"),
+                    '  TrailingData output not writable';
+            }
 
             eval { $a = $Func->(\$in, \$out, TrailingData => \@x) ;} ;
             like $@, mkErr("^$TopType: Parameter 'TrailingData' not a scalar reference"),
-                '  TrailingData output not scaral reference';
+                '  TrailingData output not scalar reference';
         }
     }
 
@@ -254,7 +290,7 @@ sub run
         my $FuncInverse = getTopFuncRef($TopTypeInverse);
 
         my @opts = ();
-        @opts = (RawInflate => 1)
+        @opts = (RawInflate => 1, UnLzma => 1)
             if $CompressClass eq 'IO::Compress::RawInflate';
 
         for my $append ( 1, 0 )
@@ -262,9 +298,8 @@ sub run
             my $already = '';
             $already = 'abcde' if $append ;
 
-            for my $buffer ( undef, '', "abcde" )
+            for my $buffer ( undef, '', $OriginalContent1 )
             {
-
                 my $disp_content = defined $buffer ? $buffer : '<undef>' ;
 
                 my $keep = $buffer;
@@ -280,8 +315,8 @@ sub run
                     is $keep, $buffer, "  Input buffer not changed" ;
                     my $got = anyUncompress(\$output, $already);
                     $got = undef if ! defined $buffer && $got eq '' ;
+                    ok ! $$Error, "  no error [$$Error]" ;
                     is $got, $buffer, "  Uncompressed matches original";
-
                 }
 
                 {
@@ -491,25 +526,26 @@ sub run
 
         my $TopTypeInverse = getInverse($bit);
         my $FuncInverse = getTopFuncRef($TopTypeInverse);
+        my $ErrorInverse = getErrorRef($TopTypeInverse);
 
         my $lex = new LexFile(my $file1, my $file2) ;
 
-        writeFile($file1, "data1");
-        writeFile($file2, "data2");
+        writeFile($file1, $OriginalContent1);
+        writeFile($file2, $OriginalContent2);
         my $of = new IO::File "<$file1" ;
         ok $of, "  Created output filehandle" ;
 
         #my @input = (   undef, "", $file2, \undef, \'', \"abcde", $of) ;
-        #my @expected = ("", "", $file2, "", "", "abcde", "data1");
-        #my @uexpected = ("", "", "data2", "", "", "abcde", "data1");
+        #my @expected = ("", "", $file2, "", "", "abcde", $OriginalContent1);
+        #my @uexpected = ("", "", $OriginalContent2, "", "", "abcde", $OriginalContent1);
         #my @input = (   $file2, \"abcde", $of) ;
-        #my @expected = ( $file2, "abcde", "data1");
-        #my @uexpected = ("data2", "abcde", "data1");
+        #my @expected = ( $file2, "abcde", $OriginalContent1);
+        #my @uexpected = ($OriginalContent2, "abcde", $OriginalContent1);
 
         my @input = (   $file1, $file2) ;
         #my @expected = ( $file1, $file2);
-        my @expected = ("data1", "data2");
-        my @uexpected = ("data1", "data2");
+        my @expected = ($OriginalContent1, $OriginalContent2);
+        my @uexpected = ($OriginalContent1, $OriginalContent2);
 
         my @keep = @input ;
 
@@ -587,9 +623,64 @@ sub run
                 my @headers = getHeaders($file3);
                 is @headers, $ms ? @input : 1, "  Header count ok";
             }
+
+            SKIP:
+            {
+                title "Truncated file";
+                skip '', 7
+                    if $CompressClass =~ /lzop|lzf|lzma/i ;
+
+                my @in ;
+                push @in, "abcde" x 10;
+                push @in, "defgh" x 1000;
+                push @in, "12345" x 50000;
+
+                my $out;
+
+                for (@in) {
+                  ok &$Func(\$_ , \$out, Append => 1 ), '  Compressed ok'
+                    or diag $$Error;
+                }
+                #ok &$Func(\@in, \$out, MultiStream => 1 ), '  Compressed ok'
+                substr($out, -179) = '';
+
+                my $got;
+                my $status ;
+                ok $status = &$FuncInverse(\$out => \$got, MultiStream => 0), "  Uncompressed stream 1 ok";
+                is $got, "abcde" x 10 ;
+                ok ! &$FuncInverse(\$out => \$got, MultiStream => 1), "  Didn't uncompress";
+                is $$ErrorInverse, "unexpected end of file", "  Got unexpected eof";
+            }
         }
     }
 
+    foreach my $bit ($CompressClass)
+    {
+
+        my $Error = getErrorRef($bit);
+        my $Func = getTopFuncRef($bit);
+        my $TopType = getTopFuncName($bit);
+
+        my $TopTypeInverse = getInverse($bit);
+        my $FuncInverse = getTopFuncRef($TopTypeInverse);
+        my $ErrorInverse = getErrorRef($TopTypeInverse);
+
+        title 'Round trip binary data that happens to include \r\n' ;
+
+        my $lex = new LexFile(my $file1, my $file2, my $file3) ;
+
+        my $original = join '', map { chr } 0x00 .. 0xff ;
+        $original .= "data1\r\ndata2\r\ndata3\r\n" ;
+
+        writeFile($file1, $original);
+        is readFile($file1), $original;
+
+        ok &$Func($file1 => $file2), '  Compressed ok' ;
+        ok &$FuncInverse($file2 => $file3), '  Uncompressed ok' ;
+        is readFile($file3), $original, "  round tripped ok";
+ 
+    }
+
     foreach my $bit ($UncompressClass,
                     #'IO::Uncompress::AnyUncompress',
                     )
@@ -839,21 +930,19 @@ sub run
         for my $files ( [qw(a1)], [qw(a1 a2 a3)] )
         {
 
-            my $tmpDir1 = 'tmpdir1';
-            my $tmpDir2 = 'tmpdir2';
+            my $tmpDir1 ;
+            my $tmpDir2 ;
             my $lex = new LexDir($tmpDir1, $tmpDir2) ;
-
-            mkdir $tmpDir1, 0777;
-            mkdir $tmpDir2, 0777;
+            my $d1 = quotemeta $tmpDir1 ;
+            my $d2 = quotemeta $tmpDir2 ;
 
             ok   -d $tmpDir1, "  Temp Directory $tmpDir1 exists";
-            #ok ! -d $tmpDir2, "  Temp Directory $tmpDir2 does not exist";
 
             my @files = map { "$tmpDir1/$_.tmp" } @$files ;
             foreach (@files) { writeFile($_, "abc $_") }
 
             my @expected = map { "abc $_" } @files ;
-            my @outFiles = map { s/$tmpDir1/$tmpDir2/; $_ } @files ;
+            my @outFiles = map { s/$d1/$tmpDir2/; $_ } @files ;
 
             {
                 title "$TopType - From FileGlob to FileGlob files [@$files]" ;
@@ -910,8 +999,7 @@ sub run
                 {
                     title "$TopType - From FileGlob to Filename files [@$files], MS $ms" ;
 
-                    my $filename = "abcde";
-                    my $lex = new LexFile($filename) ;
+                    my $lex = new LexFile(my $filename) ;
                     
                     ok &$Func("<$tmpDir1/a*.tmp>" => $filename,
                               MultiStream => $ms), '  Compressed ok' 
@@ -929,8 +1017,7 @@ sub run
                 {
                     title "$TopType - From FileGlob to Filehandle files [@$files], MS $ms" ;
 
-                    my $filename = "abcde";
-                    my $lex = new LexFile($filename) ;
+                    my $lex = new LexFile(my $filename) ;
                     my $fh = new IO::File ">$filename";
                     
                     ok &$Func("<$tmpDir1/a*.tmp>" => $fh, 
@@ -958,18 +1045,18 @@ sub run
         my $Func = getTopFuncRef($bit);
         my $TopType = getTopFuncName($bit);
 
-        my $buffer = "abcde" ;
-        my $buffer2 = "ABCDE" ;
+        my $buffer = $OriginalContent1;
+        my $buffer2 = $OriginalContent2; 
         my $keep_orig = $buffer;
 
-        my $comp = compressBuffer(getTopFuncName($UncompressClass), $buffer) ;
-        my $comp2 = compressBuffer(getTopFuncName($UncompressClass), $buffer2) ;
+        my $comp = compressBuffer($UncompressClass, $buffer) ;
+        my $comp2 = compressBuffer($UncompressClass, $buffer2) ;
         my $keep_comp = $comp;
 
         my $incumbent = "incumbent data" ;
 
         my @opts = (Strict => 1);
-        push @opts,  (RawInflate => 1)
+        push @opts,  (RawInflate => 1, UnLzma => 1)
             if $bit eq 'IO::Uncompress::AnyUncompress';
 
         for my $append (0, 1)
@@ -1264,21 +1351,21 @@ sub run
         my $buffer = "abcde" ;
         my $keep_orig = $buffer;
 
-        my $null = compressBuffer(getTopFuncName($UncompressClass), "") ;
-        my $undef = compressBuffer(getTopFuncName($UncompressClass), undef) ;
-        my $comp = compressBuffer(getTopFuncName($UncompressClass), $buffer) ;
+        my $null = compressBuffer($UncompressClass, "") ;
+        my $undef = compressBuffer($UncompressClass, undef) ;
+        my $comp = compressBuffer($UncompressClass, $buffer) ;
         my $keep_comp = $comp;
 
         my @opts = ();
-        @opts = (RawInflate => 1)
+        @opts = (RawInflate => 1, UnLzma => 1)
             if $bit eq 'IO::Uncompress::AnyUncompress';
 
         my $incumbent = "incumbent data" ;
 
         my $lex = new LexFile(my $file1, my $file2) ;
 
-        writeFile($file1, compressBuffer(getTopFuncName($UncompressClass),"data1"));
-        writeFile($file2, compressBuffer(getTopFuncName($UncompressClass),"data2"));
+        writeFile($file1, compressBuffer($UncompressClass, $OriginalContent1));
+        writeFile($file2, compressBuffer($UncompressClass, $OriginalContent2));
 
         my $of = new IO::File "<$file1" ;
         ok $of, "  Created output filehandle" ;
@@ -1286,7 +1373,7 @@ sub run
         #my @input    = ($file2, \$undef, \$null, \$comp, $of) ;
         #my @expected = ('data2', '',      '',    'abcde', 'data1');
         my @input    = ($file1, $file2);
-        my @expected = ('data1', 'data2');
+        my @expected = ($OriginalContent1, $OriginalContent2);
 
         my @keep = @input ;
 
@@ -1348,25 +1435,23 @@ sub run
         my $Func = getTopFuncRef($bit);
         my $TopType = getTopFuncName($bit);
 
-        my $tmpDir1 = 'tmpdir1';
-        my $tmpDir2 = 'tmpdir2';
+        my $tmpDir1 ;
+        my $tmpDir2 ;
         my $lex = new LexDir($tmpDir1, $tmpDir2) ;
-
-        mkdir $tmpDir1, 0777;
-        mkdir $tmpDir2, 0777;
+        my $d1 = quotemeta $tmpDir1 ;
+        my $d2 = quotemeta $tmpDir2 ;
 
         my @opts = ();
-        @opts = (RawInflate => 1)
+        @opts = (RawInflate => 1, UnLzma => 1)
             if $bit eq 'IO::Uncompress::AnyUncompress';
 
         ok   -d $tmpDir1, "  Temp Directory $tmpDir1 exists";
-        #ok ! -d $tmpDir2, "  Temp Directory $tmpDir2 does not exist";
 
         my @files = map { "$tmpDir1/$_.tmp" } qw( a1 a2 a3) ;
-        foreach (@files) { writeFile($_, compressBuffer(getTopFuncName($UncompressClass), "abc $_")) }
+        foreach (@files) { writeFile($_, compressBuffer($UncompressClass, "abc $_")) }
 
         my @expected = map { "abc $_" } @files ;
-        my @outFiles = map { s/$tmpDir1/$tmpDir2/; $_ } @files ;
+        my @outFiles = map { s/$d1/$tmpDir2/; $_ } @files ;
 
         {
             title "$TopType - From FileGlob to FileGlob" ;
@@ -1424,8 +1509,7 @@ sub run
         {
             title "$TopType - From FileGlob to Filehandle" ;
 
-            my $output = 'abc' ;
-            my $lex = new LexFile $output ;
+            my $lex = new LexFile my $output ;
             my $fh = new IO::File ">$output" ;
             ok &$Func("<$tmpDir1/a*.tmp>" => $fh, AutoClose => 1, @opts), '  UnCompressed ok' 
                 or diag $$Error ;
@@ -1534,6 +1618,49 @@ sub run
         }
     }
 
+
+    {
+        # check setting $\ 
+
+        my $CompFunc = getTopFuncRef($CompressClass);
+        my $UncompFunc = getTopFuncRef($UncompressClass);
+        my $lex = new LexFile my $file ;
+
+        local $\ = "\n" ;
+        my $input = "hello world";
+        my $compressed ;
+        my $output;
+        ok &$CompFunc(\$input => \$compressed), '  Compressed ok' ;
+        ok &$UncompFunc(\$compressed => $file), '  UnCompressed ok' ;
+        my $content = readFile($file) ;
+        is $content, $input, "round trip ok" ;
+
+    }
+
+    SKIP:
+    {
+        #95494: IO::Uncompress::Gunzip: Can no longer gunzip to in-memory file handle
+
+        skip "open filehandle to buffer not supported in Perl $]", 7
+            if $] < 5.008 ;
+        my $CompFunc = getTopFuncRef($CompressClass);
+        my $UncompFunc = getTopFuncRef($UncompressClass);
+
+        my $input = "hello world";
+        my $compressed ;
+        ok open my $fh_in1, '<', \$input ;
+        ok open my $fh_out1, '>', \$compressed ;
+        ok &$CompFunc($fh_in1 => $fh_out1), '  Compressed ok' ;
+
+        my $output;
+        ok open my $fh_in2, '<', \$compressed ;
+        ok open my $fh_out2, '>', \$output ;
+
+        ok &$UncompFunc($fh_in2 => $fh_out2), '  UnCompressed ok' ;
+        is $output, $input, "round trip ok" ;
+    }
+
+  
 }
 
 # TODO add more error cases
